/***************************************************************************
 * File:          hptchip.h
 * Description:   data structure for chip
 * Copyright (c)  2000-2003 HighPoint Technologies, Inc. All rights reserved
 ***************************************************************************/

#ifndef _CHIP_H_
#define _CHIP_H_

/***************************************************************************
 * Description:  PCI
 ***************************************************************************/

#define MAX_PCI_BUS_NUMBER      0xff  /* Maximum PCI device number */
#define MAX_PCI_DEVICE_NUMBER   0x20  /* Maximum PCI device number */
#define MAX_PCI_FUNC_NUMBER     3  /* least for ICH5 */

/*
 * PCI config Register offset
 */
#define REG_VID             0x00    /* vender identification register */
#define REG_DID             0x02    /* device identification register */
#define REG_PCICMD          0x04    /* command register */
#define REG_PCISTS          0x06    /* PCI device status register */
#define REG_RID             0x08    /* revision identification register */
#define REG_PI              0x09    /* programming interface register */
#define REG_SUBC            0x0a    /* sub class code register */
#define REG_BCC             0x0b    /* base class code register */
#define REG_MLT             0x0d    /* master latency timer register */
#define REG_HEDT            0x0e    /* header type register */
#define REG_IOPORT0         0x10    /* */
#define REG_IOPORT1         0x14    /* */
#define REG_BMIBA           0x20    /* bus master interface base address reg */
#define REG_MISC            0x50    /* brooklyn,MISC.control register */


#define PCI_IOSEN           0x01    /* Enable IO space */
#define PCI_BMEN            0x04    /* Enable IDE bus master */


/*
 * Bus Master Interface
 */

#define BMI_CMD             0       /* Bus master IDE command register offset */
#define BMI_STS             2       /* Bus master IDE status register offset */
#define BMI_DTP             4       /* Bus master IDE descriptor table */
                                    /* pointer register offset */

#define BMI_CMD_STARTREAD   9       /* Start write (read from disk) */
#define BMI_CMD_STARTWRITE  1       /* Start read (write to disk) */
#define BMI_CMD_STOP        0       /* Stop BM DMI */

#define BMI_STS_ACTIVE      1       /* RO:   bus master IDE active */
#define BMI_STS_ERROR       2       /* R/WC: IDE dma error */
#define BMI_STS_INTR        4       /* R/WC: interrupt happen */
#define BMI_STS_DRV0EN      0x20    /* R/W:  drive0 is capable of DMA xfer */
#define BMI_STS_DRV1EN      0x40    /* R/W:  drive1 is capable of DMA xfer */

/* chip function table */
struct ChipProcTable {
	void (*pfnInitialize)(PChipInstance pChip, UINT reset);
	void (*pfnModeSetting)(PChannel pChan, PDevice pDevice, UCHAR Mode);
	/* check 80-pin cable */
	void (*pfnCableDetect)(PDevice pDevice);
	/* enable/disable INTRQ. return previous state */
	UCHAR (*pfnControlIrq)(PChipInstance pChip, UCHAR enable);
	/* DMA post process */
	BOOLEAN (*pfnFinishInterrupt)(PDevice pDev);
	void (*pfnResetEngine)(PChannel pChan, UCHAR unit);
	void (*pfnResetController)(PChannel pChan);
	/* call after drives scanned */
	void (*pfnInit2)(PChipInstance pChip);
	UCHAR (*pfnWaitOnBusy)(PChannel pChan, UCHAR unit, int select);

#define TCQ_POLLING_FN(f)

#define CLOCK_SWITCH_FN(f1, f2)

#define PNP_MANAGEMENT_FN(f1, f2, f3, f4)
};

/***************************************************************************
 * Description:  Chip Instance
 ***************************************************************************/

typedef struct _ChipInstance
{
    UCHAR         bChipType;
    UCHAR         bChipIntrNum;
    UCHAR         bCpMaxChannel;
    UCHAR         bChipID;

    UCHAR         pci_bus;
	UCHAR         pci_dev;
	UCHAR         pci_func;
	UCHAR         using_dpll; /* valid only CHECK_CLOCK_SWITCH defined */

    PVBus         pVBus;

	union _uChipInstance {
		PUCHAR        _pPciIoMap;
		ULONG         _pChipCtrl;
	} u;
#define pPciIoMap u._pPciIoMap
#define pChipCtrl u._pChipCtrl

    USHORT        ChipFlags;
	UCHAR         first_on_slot;
	UCHAR         pad1;

    PChannel      pChannel[MAX_CHANNEL_PER_CHIP];

	struct ChipProcTable ftbl; /* beware BIOS data if use pointer here */


    ULONG        *Setting;


} ChipInstance;

/*
 * Chip types and flags
 * NOTE: Keep this same with definition in hptintf.h
 */
#define CHIP_TYPE_HPT366      1
#define CHIP_TYPE_HPT368      2
#define CHIP_TYPE_HPT370      3
#define CHIP_TYPE_HPT370A     4
#define CHIP_TYPE_HPT370B     5
#define CHIP_TYPE_HPT374      6
#define CHIP_TYPE_HPT372      7
#define CHIP_TYPE_HPT372A     8
#define CHIP_TYPE_HPT302      9
#define CHIP_TYPE_HPT371      10
#define CHIP_TYPE_HPT372N     11
#define CHIP_TYPE_HPT302N     12
#define CHIP_TYPE_HPT371N     13
#define CHIP_TYPE_SI3112A     14
#define CHIP_TYPE_ICH5		  15
#define CHIP_TYPE_ICH5R		  16
#define CHIP_TYPE_HPT584      17

#define CHIP_TYPE_3PARTY      0x80

/*
 * ChipFlags
 */
#define CHIP_SUPPORT_ULTRA_66   	0x20
#define CHIP_SUPPORT_ULTRA_100  	0x40
#define CHIP_HPT3XX_DPLL_MODE   	0x80
#define CHIP_DPLL_MODE CHIP_HPT3XX_DPLL_MODE
#define CHIP_SUPPORT_ULTRA_133      0x01

#define CHIP_OS_REGISTED		  	0x100
#define CHIP_LIMIT_U7               0x200 

typedef struct _CHIP_CONFIG_INFO {
	ULONG  DeviceId; /* vendor+device */
	UCHAR  Revision;
	UCHAR  ChipType;
	USHORT nBaseAddress;
	struct ChipProcTable *ftbl;
}
CHIP_CONFIG_INFO, *PCHIP_CONFIG_INFO;

extern int nChipInfo;
extern CHIP_CONFIG_INFO ChipInfoTable[];
extern PCHIP_CONFIG_INFO GetChipConfigInfo(ULONG id, UCHAR rev);

/***************************************************************************
 * Description:  Global Data
 ***************************************************************************/

UINT GetNextPciInfo(PChipInstance pChip);

extern ChipInstance  gChipInstances[];
extern ULONG setting370_75[], setting370_66[], setting370_50[], setting370_33[];

#define ACPI_REENTRY  0xA5A5
#define INIT_ENTRY    0

/* 
 * old_stat: 1: enabled, 0: disabled 
 * take care BIOS if you change it
 */
#define mChDisableIrq(pChan, old_stat) \
	old_stat = pChan->pChipInstance->ftbl.pfnControlIrq(pChan->pChipInstance, 0)
#define mChRestoreIrq(pChan, old_stat) \
	if (!old_stat) ; else pChan->pChipInstance->ftbl.pfnControlIrq(pChan->pChipInstance, 1)

/*
 * HPT Special
 */
#define HPT_VENDORID     0x1103
#define HPT370_DEVICEID  0x0004
#define HPT372A_DEVICEID 0x0005
#define HPT302_DEVICEID  0x0006
#define HPT371_DEVICEID  0x0007
#define HPT374_DEVICEID  0x0008


#endif

